@@ -39,7 +39,7 @@ def make_tf2onnx_code(opset, name=None, op_type=None, domain='',
         and following rows
     :return: code as str
     """
-    def simplify(name, kind, force=True):
+    def simplify(name, kind, force=False):
         value = None
         if (used is not None and name in used and
                 len(used[name]) == 1 and context is not None):
@@ -54,6 +54,10 @@ def make_tf2onnx_code(opset, name=None, op_type=None, domain='',
 
         if value is None and force:
             inits = context['initializers_dict']
+            if name not in inits:
+                raise RuntimeError(  # pragma: no cover
+                    "Unable to find init %r in %r value=%r." % (
+                        name, list(sorted(inits)), value))
             value = inits[name]
         if kind == 'list':
             if value is None:
@@ -61,6 +65,12 @@ def make_tf2onnx_code(opset, name=None, op_type=None, domain='',
             if len(value.shape) == 0:
                 return str(value)
             return str(list(value))
+        if kind == 'list_var':
+            if value is None:
+                return "varx[%r]" % name
+            if len(value.shape) == 0:
+                return str(value)
+            return str(list(value))
         raise NotImplementedError(
             "Unknown scenario to simplify (%r)." % kind)
 
@@ -70,7 +80,36 @@ def make_tf2onnx_code(opset, name=None, op_type=None, domain='',
             rows.append(
                 "node = GraphBuilder(ctx).make_unsqueeze("
                 "{'data': varx[%r], 'axes': %s}, return_node=True)"
-                "" % (inputs[0], simplify(inputs[1], 'list')))
+                "" % (inputs[0], simplify(inputs[1], 'list_var')))
+        else:
+            raise NotImplementedError(  # pragma: no cover
+                "Unable to create code for operator %r (opset <= 12)"
+                "." % op_type)
+    elif op_type == 'Squeeze':
+        if len(inputs) == 1:
+            rows.append(
+                "node = GraphBuilder(ctx).make_squeeze("
+                "{'data': varx[%r]}, return_node=True)"
+                "" % (inputs[0], ))
+        elif len(inputs) == 2:
+            rows.append(
+                "node = GraphBuilder(ctx).make_squeeze("
+                "{'data': varx[%r], 'axes': %s}, return_node=True)"
+                "" % (inputs[0], simplify(inputs[1], 'list_var')))
+        else:
+            raise NotImplementedError(  # pragma: no cover
+                "Unable to create code for operator %r (opset <= 12)"
+                "." % op_type)
+    elif op_type == 'Slice':
+        atts = dict(zip(['starts', 'ends', 'axes', 'steps'],
+                        inputs[1:]))
+        text = ", ".join("'%s': %s" % (k, simplify(v, 'list_var'))
+                         for k, v in atts.items())
+        if len(inputs) in (3, 4, 5):
+            rows.append(
+                "node = GraphBuilder(ctx).make_slice("
+                "{'data': varx[%r], %s}, return_node=True)"
+                "" % (inputs[0], text))
         else:
             raise NotImplementedError(  # pragma: no cover
                 "Unable to create code for operator %r (opset <= 12)"
@@ -105,14 +144,21 @@ def make_name(name):
     return name
 
 
+def get_max_value(np_dtype):
+    "Returns the maximum value for a specific type."
+    return numpy.iinfo(np_dtype).max
+
+
 def make_sure(cond, msg, *args):
     "Raises an exception if cond is not verified."
     if not cond:
-        raise RuntimeError(msg % tuple(args))
+        raise RuntimeError(msg % tuple(args))  # pragma: no cover
 
 
 def map_onnx_to_numpy_type(onnx_dtype):
     "Converts ONNX type into numpy type."
+    if onnx_dtype is None:
+        return numpy.float32
     return guess_dtype(onnx_dtype)
 
 
@@ -162,10 +208,11 @@ class Tf2OnnxConvert:
     """
 
     def __init__(self, onnx_model, _tf_op=None, verbose=None,
-                 target_opset=None):
+                 target_opset=None, max_iter=5):
         self._onnx_model = onnx_model
         self._tf_op = _tf_op or tf_op
         self.verbose = verbose
+        self.max_iter = max_iter
         if isinstance(target_opset, int):
             self.target_opsets = {'': target_opset}
         elif isinstance(target_opset, dict):
@@ -203,7 +250,7 @@ class Tf2OnnxConvert:
         :return: node name
         """
         if name not in self._names:
-            raise RuntimeError(
+            raise RuntimeError(  # pragma: no cover
                 "Unable to find node name %r among %r." % (
                     name, ", ".join(sorted(self._names))))
         return self._names[name]
@@ -214,13 +261,14 @@ class Tf2OnnxConvert:
         :param name: node or initializer
         """
         if obj.name in self._forbidden_new_names:
-            raise RuntimeError(
+            raise RuntimeError(  # pragma: no cover
                 "Name %r is already registered." % obj.name)
         self._names[obj.name] = obj
         self._forbidden_new_names.add(obj.name)
 
     def make_node(self, op_type, inputs, attr=None, outputs=None,
-                  name=None, domain='', output_count=1):
+                  name=None, domain='', output_count=1,
+                  shapes=None, dtypes=None):
         """
         Adds a node to the list of nodes.
 
@@ -232,18 +280,21 @@ class Tf2OnnxConvert:
             the number of outputs of this node
         :param name: name of the node
         :param domain: domain
+        :param shapes: unused
+        :param dtypes: unused
         :return: created node
         """
         if self.verbose:
-            print("[Tf2OnnxConvert.make_node] op_type=%r inputs=%r" % (
-                op_type, inputs))
+            print(  # pragma: no cover
+                "[Tf2OnnxConvert.make_node] op_type=%r inputs=%r" % (
+                    op_type, inputs))
 
         if attr is None:
             attr = {}
         if name is None:
             name = make_name(op_type)
         if name in self._names:
-            raise RuntimeError(
+            raise RuntimeError(  # pragma: no cover
                 "Node name %r already exists in %r." % (
                     name, ", ".join(sorted(self._names))))
 
@@ -278,7 +329,7 @@ class Tf2OnnxConvert:
         :return: create initializer
         """
         if name in self._names:
-            raise RuntimeError(
+            raise RuntimeError(  # pragma: no cover
                 "Initializer name %r already exists in %r." % (
                     name, ", ".join(sorted(self._names))))
         np_val_flat = np_val.flatten()
@@ -316,19 +367,24 @@ class Tf2OnnxConvert:
         :param new_name: new name
         :return: list of impacted nodes
         """
+        if self.verbose:
+            print(  # pragma: no cover
+                "[Tf2OnnxConvert.replace_all_inputs] replace %r by %r" % (
+                    old_name, new_name))
         res = []
         for node in self._names.values():
             if not hasattr(node, 'input'):
                 continue
             if old_name not in node.input:
                 continue
-            new_inputs = [new_name if i.name == old_name else i.name
+            new_inputs = [new_name if i == old_name else i
                           for i in node.input]
             node.input[:] = new_inputs[:]
             res.append(node)
             if self.verbose:
-                print("[Tf2OnnxConvert.replace_all_inputs] replace %r by %r in node %r" % (
-                    old_name, new_name, node.name))
+                print(  # pragma: no cover
+                    "[Tf2OnnxConvert.replace_all_inputs] replace %r by %r in node %r" % (
+                        old_name, new_name, node.name))
         for o in self._onnx_model.graph.output:
             if o.name != old_name:
                 continue
@@ -336,9 +392,13 @@ class Tf2OnnxConvert:
                                name=make_name("IdOutputReplaced"))
             res.append(n)
             if self.verbose:
-                print("[Tf2OnnxConvert.replace_all_inputs] add id node from %r to %r "
-                      "with node %r." % (
-                          old_name, new_name, n.name))  # pylint: disable=E1101
+                print(  # pragma: no cover
+                    "[Tf2OnnxConvert.replace_all_inputs] add id node from %r to %r "
+                    "with node %r." % (
+                        old_name, new_name, n.name))  # pylint: disable=E1101
+        if self.verbose:
+            print(  # pragma: no cover
+                "[Tf2OnnxConvert.replace_all_inputs] end")
         return res
 
     def remove_node(self, name):
@@ -346,11 +406,12 @@ class Tf2OnnxConvert:
         Removes a node name from the list.
         """
         if name not in self._names:
-            raise RuntimeError(
+            raise RuntimeError(  # pragma: no cover
                 "Unable to delete name %r because it does not exists." % name)
         del self._names[name]
         if self.verbose:
-            print("[Tf2OnnxConvert.remove_node] delete name %r" % name)
+            print(  # pragma: no cover
+                "[Tf2OnnxConvert.remove_node] delete name %r" % name)
 
     def get_shape(self, input_name):
         """
@@ -377,12 +438,14 @@ class Tf2OnnxConvert:
             raise RuntimeError(  # pragma: no cover
                 "No converter was registered.")
         if self.verbose:
-            print("[Tf2OnnxConvert.run]")
+            print("[Tf2OnnxConvert.run]")  # pragma: no cover
 
         done = {}
         modif = 1
-        while modif > 0:
+        turn = 0
+        while modif > 0 and turn < self.max_iter:
             modif = 0
+            turn += 1
             # The converter may alter the current list of nodes, we freeze it.
             current_values = list(self._names.values())
             for node in current_values:
@@ -408,12 +471,17 @@ class Tf2OnnxConvert:
 
                 # applies the converter
                 if self.verbose:
-                    print("[Tf2OnnxConvert.run] convert node type=%r opset=%r name=%r"
-                          "" % (node.op_type, target, node.name))
+                    print(  # pragma: no cover
+                        "[Tf2OnnxConvert.run] convert node type=%r opset=%r name=%r"
+                        "" % (node.op_type, target, node.name))
                 fct, kwargs = conv
                 fct(self, node, target_opset=target, **kwargs)
                 modif += 1
 
+        if turn >= self.max_iter:
+            raise RuntimeError(  # pragma: no cover
+                "Too many iterations and no stable ONNX was reached, "
+                "iter=%d\n%s" % (turn, str(self.make_model())))
         return self.make_model()
 
     def make_model(self):
@@ -429,7 +497,7 @@ class Tf2OnnxConvert:
         nodes = ensure_topological_order(inputs, inits, nodes)
 
         if self.verbose:
-            print(
+            print(  # pragma: no cover
                 "[Tf2OnnxConvert.make_node] %d nodes %d inputs %d "
                 "outputs %d initializers"
                 "" % (len(nodes), len(inputs), len(outputs), len(inits)))
@@ -468,18 +536,22 @@ class GraphBuilder:
         "Returns the graph."
         return self._g
 
-    def make_slice(self, kwargs, name=None, shapes=None, dtypes=None, return_node=False):
+    def make_slice(self, kwargs, name=None, shapes=None, dtypes=None,
+                   return_node=False):
         """
-        slice changes its schema at opset 10: it treats some attributes as dynamic input
-        so this function has to process inputs according to graph's opset version
+        slice changes its schema at opset 10: it treats some
+        attributes as dynamic input so this function has to process
+        inputs according to graph's opset version
         to get "inputs" and "attr" to feed "make_node"
-        kwargs: key could be ["data", "starts", "ends", "axes", "steps", "outputs"].
+        kwargs: key could be `["data", "starts", "ends",
+        "axes", "steps", "outputs"]`.
         """
         outputs = kwargs.pop("outputs", None)
 
         if self.graph.opset < 10:
             # "data" is string
-            # "starts", "ends" and "axes" are attributes, and "axes" is optional.
+            # "starts", "ends" and "axes" are attributes,
+            # and "axes" is optional.
             data = kwargs.pop("data")
             starts = self._convert_to_attribute(kwargs.pop("starts"))
             ends = self._convert_to_attribute(kwargs.pop("ends"))
@@ -490,18 +562,21 @@ class GraphBuilder:
         else:
             # slice-10 has 3 required inputs "data", "starts", "ends"l
             # and 2 optional inputs "axes", "steps"
-            # input sequence should be "data", "starts", "ends", "axes", "steps"
+            # input sequence should be "data", "starts", "ends",
+            # "axes", "steps"
             attr = {}
             data = kwargs.pop("data")
-            starts = self._convert_to_input(kwargs.pop(
-                "starts"), "const_starts", dtype=numpy.int64)
-            ends = self._convert_to_input(kwargs.pop(
-                "ends"), "const_ends", dtype=numpy.int64)
-            axes = self._convert_to_input(kwargs.pop(
-                "axes", None), "const_axes", is_optional=True, dtype=numpy.int64)
-            steps = self._convert_to_input(kwargs.pop(
-                "steps", None), "const_steps", is_optional=True, dtype=numpy.int64)
-            inputs = [data, starts.name, ends.name, axes.name, steps.name]
+            starts = self._convert_to_input(
+                kwargs.pop("starts"), "const_starts", dtype=numpy.int64)
+            ends = self._convert_to_input(
+                kwargs.pop("ends"), "const_ends", dtype=numpy.int64)
+            axes = self._convert_to_input(
+                kwargs.pop("axes", None), "const_axes",
+                is_optional=True, dtype=numpy.int64)
+            steps = self._convert_to_input(
+                kwargs.pop("steps", None), "const_steps",
+                is_optional=True, dtype=numpy.int64)
+            inputs = [data, starts, ends, axes, steps]
 
         # pro-process inputs and attr
         make_sure(not kwargs, "kwargs contains un-used key")
@@ -531,7 +606,8 @@ class GraphBuilder:
                                     dtypes=dtypes)
         if return_node:
             return node
-        raise NotImplementedError("return_node must be True")
+        raise NotImplementedError(  # pragma: no cover
+            "return_node must be True")
 
     def make_squeeze(self, kwargs, name=None, shapes=None, dtypes=None,
                      return_node=False, op_name_scope=None):
@@ -549,10 +625,11 @@ class GraphBuilder:
             inputs = [data]
         else:
             data = kwargs.pop("data")
-            axes = self._convert_to_input(kwargs.pop(
-                "axes", None), "const_axes", is_optional=True, dtype=numpy.int64)
+            axes = self._convert_to_input(
+                kwargs.pop("axes", None), "const_axes",
+                is_optional=True, dtype=numpy.int64)
             attr = {}
-            inputs = [data, axes.name]
+            inputs = [data, axes]
 
         make_sure(not kwargs, "kwargs contains un-used key")
 
@@ -574,7 +651,8 @@ class GraphBuilder:
             outputs=outputs)
         if return_node:
             return node
-        raise NotImplementedError("return_node must be True")
+        raise NotImplementedError(  # pragma: no cover
+            "return_node must be True")
 
     def make_unsqueeze(self, kwargs, name=None, shapes=None, dtypes=None,
                        return_node=False, op_name_scope=None):
@@ -592,10 +670,11 @@ class GraphBuilder:
             inputs = [data]
         else:
             data = kwargs.pop("data")
-            axes = self._convert_to_input(kwargs.pop(
-                "axes", None), "const_axes", is_optional=True, dtype=numpy.int64)
+            axes = self._convert_to_input(
+                kwargs.pop("axes", None), "const_axes",
+                is_optional=True, dtype=numpy.int64)
             attr = {}
-            inputs = [data, axes.name]
+            inputs = [data, axes]
 
         make_sure(not kwargs, "kwargs contains un-used key")
 
@@ -617,7 +696,8 @@ class GraphBuilder:
             outputs=outputs)
         if return_node:
             return node
-        raise NotImplementedError("return_node must be True")
+        raise NotImplementedError(  # pragma: no cover
+            "return_node must be True")
 
     def _convert_to_input(self, tensor, const_name, is_optional=False, dtype=None):
         """in ONNX, input shold come from node, so it must be a string"""
@@ -630,7 +710,7 @@ class GraphBuilder:
         res = tensor
         if isinstance(tensor, list):
             res = self.graph.make_const(
-                make_name(const_name), numpy.array(tensor, dtype))
+                make_name(const_name), numpy.array(tensor, dtype)).name
         return res
 
     def _convert_to_attribute(self, tensor, is_optional=False):
