@@ -66,19 +66,17 @@ class OpRun:
         self.desc = desc
         self.inplaces = {}
 
-        if '_' in self.__class__.__name__:
-            self._schema = _schemas.get(self.__class__.__name__, None)
-            if self._schema is None:
-                raise RuntimeError(  # pragma: no cover
-                    "Unable to find class name '{}' in available schemas:"
-                    "(onnx.__version__='{}')\n{}".format(
-                        self.__class__.__name__,
-                        onnx.__version__,
-                        "\n".join(sorted(_schemas))))
-        elif onnx_node.op_type in _schemas:
+        if onnx_node.op_type in _schemas:
             self._schema = _schemas[onnx_node.op_type]
         else:
             self._schema = self._find_custom_operator_schema(onnx_node.op_type)
+        if self._schema is None:
+            raise RuntimeError(  # pragma: no cover
+                "Unable to find class name '{}' in available schemas:"
+                "(onnx.__version__='{}')\n{}".format(
+                    self.__class__.__name__,
+                    onnx.__version__,
+                    "\n".join(sorted(_schemas))))
 
         if desc is not None:
             if 'atts' in desc:
@@ -679,8 +677,10 @@ class OpRunBinaryNum(OpRunBinary):
         res = OpRunBinary.run(self, x, y)
         if res[0].dtype != x.dtype:
             raise RuntimeTypeError(
-                "Output type mismatch: {} != {} (operator '{}')".format(
-                    x.dtype, res[0].dtype, self.__class__.__name__))
+                "Output type mismatch: {} != {} or {} (operator '{}')"
+                " type(x)={} type(y)={}".format(
+                    x.dtype, res[0].dtype, y.dtype,
+                    self.__class__.__name__, type(x), type(y)))
         return res
 
     def _run_no_checks_(self, x, y):  # pylint: disable=W0221
