@@ -592,7 +592,9 @@ def _call_runtime(obs_op, conv, opset, debug, inst, runtime,
         fLOG("[enumerate_compatible_opset-R] load onnx")
     try:
         sess, t5, ___ = _measure_time(
-            lambda: OnnxInference(ser, runtime=runtime))
+            lambda: OnnxInference(
+                ser, runtime=runtime, runtime_options=dict(
+                    log_severity_level=3)))
         obs_op['tostring_time'] = t5
     except (RuntimeError, ValueError, KeyError, IndexError, TypeError) as e:
         if debug:
@@ -670,11 +672,17 @@ def _call_runtime(obs_op, conv, opset, debug, inst, runtime,
                     opred.shape[1] == 2 and len(ypred.shape) == 1):
                 # decision_function, for binary classification,
                 # raw score is a distance
-                max_rel_diff = measure_relative_difference(
-                    ypred, opred[:, 1])
+                try:
+                    max_rel_diff = measure_relative_difference(
+                        ypred, opred[:, 1])
+                except AttributeError:
+                    max_rel_diff = numpy.nan
             else:
-                max_rel_diff = measure_relative_difference(
-                    ypred, opred)
+                try:
+                    max_rel_diff = measure_relative_difference(
+                        ypred, opred)
+                except AttributeError:
+                    max_rel_diff = numpy.nan
 
             if max_rel_diff >= 1e9 and debug:  # pragma: no cover
                 _shape = lambda o: o.shape if hasattr(
